@@ -1,8 +1,14 @@
 """Test module for GasOil objects"""
+import io
 
 import numpy as np
 import pandas as pd
 
+import matplotlib
+import matplotlib.pyplot
+
+import pytest
+
 from hypothesis import given, settings
 import hypothesis.strategies as st
 
@@ -18,9 +24,7 @@ from pyscal.utils.testing import (
 
 
 def test_gasoil_init():
-    """Check the __init__ method for GasOil
-
-    are arguments handled correctly?"""
+    """Test features in the constructor"""
     gasoil = GasOil()
     assert isinstance(gasoil, GasOil)
     assert gasoil.swirr == 0.0
@@ -60,6 +64,29 @@ def test_gasoil_init():
     assert np.isclose(gasoil.table["sg"].min(), 0)
     assert np.isclose(gasoil.table["sg"].max(), 0.9)
 
+    # Test too small h:
+    gasoil = GasOil(swl=0.1, h=0.00000000000000000001)
+    # (a warning is printed that h is truncated)
+    assert gasoil.h == 1 / SWINTEGERS
+
+
+def test_errors():
+    """Test some error situations for the constructor"""
+    with pytest.raises(ValueError, match="No saturation range left"):
+        GasOil(swl=0.3, sorg=0.8)
+
+    with pytest.warns(DeprecationWarning):
+        gasoil = GasOil(tag=dict())
+    assert gasoil.tag == ""
+
+
+def test_plotting():
+    """Test that plotting code pass through (nothing displayed)"""
+    gasoil = GasOil(swl=0.1, h=0.1)
+    gasoil.add_corey_gas()
+    gasoil.add_corey_oil()
+    gasoil.plotkrgkrog(mpl_ax=matplotlib.pyplot.subplots()[1])
+
 
 @settings(deadline=300)
 @given(st.text())
@@ -244,6 +271,27 @@ def test_gasoil_krgendanchor():
     assert gasoil.table[np.isclose(gasoil.table["sg"], 1.0)]["krg"].values[0] == 1.0
 
 
+def test_nexus():
+    """Test the Nexus export"""
+    gasoil = GasOil(h=0.01, swl=0.1, sgcr=0.3, sorg=0.3)
+    gasoil.add_corey_oil(nog=10, kroend=0.5)
+    gasoil.add_corey_gas(ng=10, krgend=0.5)
+    nexus_lines = gasoil.GOTABLE().splitlines()
+    non_comments = [
+        line for line in nexus_lines if not line.startswith("!") or not len(line)
+    ]
+    assert non_comments[0] == "GOTABLE"
+    assert non_comments[1] == "SG KRG KROG PC"
+    df = pd.read_table(
+        io.StringIO("\n".join(non_comments[2:])),
+        engine="python",
+        sep=r"\s+",
+        header=None,
+    )
+    assert (df.values <= 1.0).all()
+    assert (df.values >= 0.0).all()
+
+
 def test_linearsegments():
     """Made for testing the linear segments during
     the resolution of issue #163"""
@@ -252,7 +300,6 @@ def test_linearsegments():
     gasoil.add_corey_gas(ng=10, krgend=0.5)
     check_table(gasoil.table)
     check_linear_sections(gasoil)
-    # gasoil.plotkrgkrog(marker="*")
 
 
 def test_kroend():
@@ -372,6 +419,42 @@ def test_sgfn():
     print(sgfn_str)
 
 
+def test_fast():
+    """Test the fast option"""
+    # First without fast-mode:
+    gasoil = GasOil(h=0.1)
+    gasoil.add_corey_gas()
+    gasoil.add_corey_oil()
+    # This crosspoint computation is only present for fast=False:
+    assert "-- krg = krog @ sg=0.5" in gasoil.SGOF()
+
+    # Provoke non-strict-monotone krow:
+    gasoil.table.loc[0:2, "krog"] = [1.00, 0.81, 0.81]
+    # (this is valid in non-imbibition, but pyscal will correct it for all
+    # curves)
+    assert "0.1000000 0.0100000 0.8100000 0.0000000" in gasoil.SGOF()
+    assert "0.2000000 0.0400000 0.8099999 0.0000000" in gasoil.SGOF()
+    #   monotonocity correction:   ^^^^^^
+
+    # Now redo with fast option:
+    gasoil = GasOil(h=0.1, fast=True)
+    gasoil.add_corey_gas()
+    gasoil.add_corey_oil()
+    # This crosspoint computation is only present for fast=False:
+    assert "-- krg = krog" not in gasoil.SGOF()
+
+    # Provoke non-strict-monotone krow, in fast-mode
+    # this slips through:
+    gasoil.table.loc[0:2, "krog"] = [1.00, 0.81, 0.81]
+    assert "0.1000000 0.0100000 0.8100000 0.0000000" in gasoil.SGOF()
+    assert "0.2000000 0.0400000 0.8100000 0.0000000" in gasoil.SGOF()
+    # not corrected:               ^^^^^^
+
+    gasoil.table.loc[0:2, "krg"] = [0.00, 0.01, 0.01]
+    assert "0.1000000 0.0100000" in gasoil.SGFN()
+    assert "0.2000000 0.0100000" in gasoil.SGFN()
+
+
 def test_roundoff():
     """Test robustness to monotonicity issues arising from
     representation errors
